function [VAFbest,Pbest,Qbest,Vbest,Xfit]=main(X,K,L)
%This function clusters a data matrix X into K row clusters and L column
%clusters, using the method of two-mode partitioning.

%This function uses the five optimization methods discussed in Van Rosmalen,
%Groenen, Trejos, and Castillo, Journal of Classification (forthcoming)
%The results for the best solution found by these five methods are
%reported by this function.
%The outputs are:
%VAFbest    The variance accounted for ratio of the best solution found
%Pbest      The matrix P of the best solution found
%Qbest      The matrix Q of the best solution found
%Vbest      The matrix V of the best solution found
%Xfit       The fitted values of the best solution found (that is, the best
%possible approximation of X)


[n,m]=size(X);
%construct random original P and Q
[P,Q]=randompartition(m,n,K,L,0);
% update V
V=P'*X*Q./(sum(P)'*sum(Q));
%  calculate variance of data for fast calculation of VAFs
variance=sum(sum((X-sum(sum(X))/(n*m)).^2));
%improve these using kmeans algorithm
[vafvalue,Ptemp,Qtemp]=kmeans(P,Q,X,m,n,K,L,variance);

%perform the five algorithms on P,Q and V and store the
%VAFs and computation times in results
 
nmultistart=30;
[vafvalue,Palg,Qalg]=multiAE(X,m,n,K,L,variance,nmultistart);
Pbest=Palg;
Qbest=Qalg;
VAFbest=vafvalue;
 
nmultistart=500;
[vafvalue,Palg,Qalg]=multikmeans(X,m,n,K,L,variance,nmultistart);
if vafvalue>VAFbest
    VAFbest=vafvalue;
    Pbest=Palg;
    Qbest=Qalg;
end

temperature=1;
gamma=0.95;
maxiter=round(.5*(n*(K-1)+m*(L-1)));
maxnnotfrozen=10;
[vafvalue,Palg,Qalg]=simanneal(Ptemp,Qtemp,X,m,n,K,L,variance,temperature,gamma,maxiter,maxnnotfrozen);
if vafvalue>VAFbest
    VAFbest=vafvalue;
    Pbest=Palg;
    Qbest=Qalg;
end

tabulength=round(1*(sqrt(n*(K-1)+m*(L-1))));
maxiter=round(3*(sqrt(n*(K-1)+m*(L-1))));
[vafvalue,Palg,Qalg]=tabusearch(Ptemp,Qtemp,X,m,n,K,L,variance,tabulength,maxiter);
if vafvalue>VAFbest
    VAFbest=vafvalue;
    Pbest=Palg;
    Qbest=Qalg;
end

s=1.05;
gamma=0.85;
s_min=1.001;
tic;
[vafvalue,Palg,Qalg]=multifuzzy(X,m,n,K,L,variance,s,gamma,s_min,20);
if vafvalue>VAFbest
    VAFbest=vafvalue;
    Pbest=Palg;
    Qbest=Qalg;
end
Vbest=Pbest'*X*Qbest./(sum(Pbest)'*sum(Qbest));
Xfit=Pbest*Vbest*Qbest';
