function [P,Q]=randompartition(m,n,K,L,fraconecluster)
%This function generates a random partition.
%The input fraconecluster is the fraction of objects in the first cluster, with
%a random distribution if fraconeclusters==0.

%generate random P and Q, so that no cluster is empty
if fraconecluster==0
    %Divide objects over clusters with uniform probability
    P=zeros(n,K);
    temp=ceil(K*rand(n,1));
    for r=1:n
        P(r,temp(r))=1;
    end
    Q=zeros(m,L);
    temp=ceil(L*rand(m,1));
    for r=1:m
        Q(r,temp(r))=1;
    end
    for j=find(sum(P)==0)
        i=ceil(n*rand);
        while sum(P(:,find(P(i,:))))<2
            i=ceil(n*rand);
        end
        P(i,:)=0;
        P(i,j)=1;
    end
    for j=find(sum(Q)==0)
        i=ceil(m*rand);
        while sum(Q(:,find(Q(i,:))))<2
            i=ceil(m*rand);
        end
        Q(i,:)=0;
        Q(i,j)=1;
    end
else
    %Ensure that one cluster receives 100*fraconecluster% of the objects
    tempP=zeros(n,K);
    tempQ=zeros(m,L);
    while any(sum(tempP)==0)
        tempP=zeros(n,K);
        tempP(1:round(fraconecluster*n),1)=1;
        temp=ceil(ones(n,1)+(K-1)*rand(n,1));
        for r=round(fraconecluster*n)+1:n
            tempP(r,temp(r))=1;
        end
    end
    while any(sum(tempQ)==0)
        tempQ=zeros(m,L);
        tempQ(1:round(fraconecluster*m),1)=1;
        temp=ceil(ones(m,1)+(L-1)*rand(m,1));
        for r=round(fraconecluster*m)+1:m
            tempQ(r,temp(r))=1;
        end
    end
    %Reorder the rows of P and Q in a random way
    permutation=randperm(n);
    for i=1:n
        P(i,:)=tempP(permutation(i),:);
    end
    permutation=randperm(m);
    for i=1:m
        Q(i,:)=tempQ(permutation(i),:);
    end
end
