function [bestcriterion,Pbest,Qbest]=tabusearch(P,Q,X,m,n,K,L,variance,tabulength,maxiter)
%This function performs the tabu search optimization method for two-mode partitioning

%initialization
eps=10^-12;
V=P'*X*Q./(sum(P)'*sum(Q));
bestcriterion=1-sum(sum((X-P*V*Q').^2))/variance;
criterion=bestcriterion;
Pbest=P;
Qbest=Q;
tabulistP=zeros(n,K,tabulength);
tabulistQ=zeros(m,L,tabulength);
tabulist=zeros(tabulength,1);
for numiter=1:maxiter
    aspcriterion=0;
    sumP=sum(P);
    sumQ=sum(Q);
    %evaluate part of neighbourhood by transferring objects in P
    for rowP=1:n
        columnPold=P(rowP,:)*(1:K)';
        if  sumP(columnPold)>1
            tempP=P;
            temp=X(rowP,:)*Q;
            tempP(rowP,columnPold)=0;
            XQdivsumQ=temp./sumQ;
            Voldclus=V(columnPold,:);
            cardold=sumP(columnPold);
            Vtempoldclus=Voldclus*cardold-XQdivsumQ;
            Vtempoldclus=Vtempoldclus/(cardold-1);
            for columnP=1:K
                if  columnP~=columnPold
                    %transfer object
                    tempP(rowP,columnP)=1;
                    %update V
                    Vnewclus=V(columnP,:);                    
                    cardnew=sumP(columnP);
                    Vtempnewclus=Vnewclus*cardnew+XQdivsumQ;
                    Vtempnewclus=Vtempnewclus/(cardnew+1);
                    % update criterion
                    newcriterion=criterion-(sumQ*(cardnew*(Vnewclus-Vtempnewclus).^2+ cardold*(Voldclus-Vtempoldclus).^2)'...
                        +(Vtempoldclus-Vtempnewclus)*(2*temp-sumQ.*(Vtempnewclus+Vtempoldclus))')/variance;
                    if any(abs(tabulist-newcriterion)<eps)
                        for i=1:tabulength
                            if abs(newcriterion-tabulist(i))<eps
                                if all(all(tabulistP(:,:,i)==tempP)) && all(all(tabulistQ(:,:,i)==Q))
                                   	newcriterion=-1;
                                end
                            end
                        end
                    end
                    if  newcriterion>aspcriterion
                        aspcriterion=newcriterion;
                        Pasp=tempP;
                        Qasp=Q;
                        Vasp=V;
                        Vasp(columnP,:)=Vtempnewclus;
                        Vasp(columnPold,:)=Vtempoldclus;
                    end
                    tempP(rowP,columnP)=0;
                end
            end
        end
    end
    %evaluate part of neighbourhood by transferring objects in Q
    for rowQ=1:m
        columnQold=Q(rowQ,:)*(1:L)';
        if  sumQ(columnQold)>1
            tempQ=Q;
            temp=P'*X(:,rowQ);            
            tempQ(rowQ,columnQold)=0;
            PtXdivsumP=temp./sumP';
            Voldclus=V(:,columnQold);
            cardold=sumQ(columnQold);
            Vtempoldclus=Voldclus*cardold- PtXdivsumP;
            Vtempoldclus=Vtempoldclus/(cardold-1);
            for columnQ=1:L
                if  columnQ~=columnQold
                    tempQ(rowQ,columnQ)=1;
                    Vnewclus=V(:,columnQ);
                    cardnew=sumQ(columnQ);
                    Vtempnewclus=Vnewclus*cardnew+ PtXdivsumP;
                    Vtempnewclus=Vtempnewclus/(cardnew+1);
                    newcriterion=criterion-(sumP*(cardnew*(Vnewclus-Vtempnewclus).^2+ cardold*(Voldclus-Vtempoldclus).^2)...
                        +(Vtempoldclus-Vtempnewclus)'*(2*temp-sumP(:).*(Vtempnewclus+Vtempoldclus)))/variance;
                    if any(abs(tabulist-newcriterion)<eps)
                        for i=1:tabulength
                            if abs(newcriterion-tabulist(i))<eps
                                if all(all(tabulistP(:,:,i)==P)) && all(all(tabulistQ(:,:,i)==tempQ))
                                    newcriterion=-1;
                                end
                            end
                        end
                    end
                    if  newcriterion>aspcriterion
                        aspcriterion=newcriterion;
                        Pasp=P;
                        Qasp=tempQ;
                        Vasp=V;
                        Vasp(:,columnQ)=Vtempnewclus;
                        Vasp(:,columnQold)=Vtempoldclus;
                    end
                    tempQ(rowQ,columnQ)=0;
                end
            end
        end
    end
    %update criterion values
    if aspcriterion>bestcriterion+eps
        Pbest=Pasp;
        Qbest=Qasp;
        bestcriterion=aspcriterion;
    end
    P=Pasp;
    Q=Qasp;
    V=Vasp;
    criterion=aspcriterion;
    %update tabu list
    for i=1:tabulength-1
        tabulistP(:,:,i)=tabulistP(:,:,i+1);
        tabulistQ(:,:,i)=tabulistQ(:,:,i+1);
        tabulist(i)=tabulist(i+1);
    end
    tabulistP(:,:,tabulength)=P;
    tabulistQ(:,:,tabulength)=Q;
    tabulist(tabulength)=aspcriterion;
end