function [vaf,P,Q]=fuzzy(P,Q,X,m,n,K,L,variance,s,gamma,s_min)
%This function performs the fuzzy optimization method for two-mode partitioning

numeps=10^-7;
%initialize V
Ps=P.^s;
Qs=Q.^s;
V=Ps'*X*Qs./(sum(Ps)'*sum(Qs));
XreplK=reshape(permute(repmat(X(:,:),[1 1 K]),[2 3 1]),[m K*n]);
XreplL=reshape(permute(repmat(X(:,:),[1 1 L]),[1 3 2]),[n L*m]);
XreplKsquared=sum((permute(repmat(X(:,:),[1 1 K]),[1 3 2])).^2,3);
XreplLsquared=sum((permute(repmat(X(:,:),[1 1 L]),[2 3 1])).^2,3);
initial_s=1;
Qold=0;
Pold=0;
onesm1 =ones(m,1);
ones1n=ones(1,n);
onesn1=ones1n';
ones1m =onesm1';
ones1K=ones(1,K);
ones1L=ones(1,L);
while s>s_min && (initial_s==1 || (min(max(P(:),1-P(:)))<0.9 && min(max(Q(:),1-Q(:)))<0.9))
    initial_s=0;
    %update s_prime
    s_prime=1/(1-s);
    numiter=0;
    Qs=Q.^s;
    while numiter<20 && (numiter==0 || ((mean(abs(Pold(:)-P(:)))>0.001) || (mean(abs(Qold(:)-Q(:)))>0.001)))
        numiter=numiter+1;
        Pold=P;
        Qold=Q;
        %update P
        dQV=zeros(m,K);

        for b=1:L
            dQV=dQV+Qs(:,b*ones1K).*(onesm1*V(:,b)');
        end
        dQV=dQV(:);
        dQVrepln=reshape(dQV(:,ones1n),[m K*n]);
        dXP=reshape(sum(dQVrepln.*XreplK,1),[K n]);
        d=-2*dXP'+XreplKsquared+onesn1*(V.^2*sum(Qs,1)')';
        logds_prime=log(d)*s_prime;
        maxlogds_prime=max(logds_prime,[],2);
        ds_prime=exp(logds_prime-maxlogds_prime(:,ones1K));
        ds_prime(ds_prime<numeps)=0;
        if all(d(:)>0)
            P=ds_prime./(sum(ds_prime,2)*ones1K);
        else
            for i=1:n
                sumd=sum(ds_prime(i,:));
                if ~any(d(i,:)==0)
                    P(i,:)=ds_prime(i,:)/sumd;
                else
                    for j=1:K
                        if d(i,j)>0
                            P(i,j)=0;
                        else
                            P(i,j)=1/sum(d(i,:)==0);
                        end
                    end
                end
            end
        end
        %fill empty clusters in P
        if any(sum(P)<numeps)
            dQV=zeros(m,K);
            for b=1:L
                dQV=dQV+Qs(:,b*ones1K).*(onesm1*V(:,b)');
            end
            dQV=dQV(:);
            dQVrepln=reshape(dQV(:,ones1n),[m L*n]);
            dXP=reshape(sum(dQVrepln.*XreplK,1),[L n]);
            d=-2*dXP'+XreplKsquared+onesn1*(V.^2*sum(Qs,1)')';
            while any(sum(P)<numeps)
                distance=sum(P.*d,2);
                for k=1:K
                    Max=-1;
                    for i=1:n
                        if distance(i)>Max && all(-P(i,:)+sum(P)+(sum(P)<numeps)>numeps)
                            maxindex=i;
                            Max=distance(i);
                        end
                    end
                    distance(maxindex)=0;
                    P(maxindex,:)=0;
                    P(maxindex,k)=1;
                end
            end
        end
        %update Q
        Ps=P.^s;
        dPV=zeros(n,L);
        for b=1:K
            dPV=dPV+Ps(:,b*ones1L).*V(b*onesn1,:);
        end
        dPV=dPV(:);
        dPVreplm=reshape(dPV(:,ones1m),[n L*m]);
        dXQ=reshape(sum(dPVreplm.*XreplL,1),[L m]);
        d=-2*dXQ'+XreplLsquared+onesm1*(sum(Ps,1)*V.^2);
        logds_prime=log(d)*s_prime;
        maxlogds_prime=max(logds_prime,[],2);
        ds_prime=exp(logds_prime-maxlogds_prime(:,ones1L));
        ds_prime(ds_prime<numeps)=0;
        if all(d(:)>0)
            Q=ds_prime./(sum(ds_prime,2)*ones1L);
        else
            for i=1:m
                somd=sum(ds_prime(i,:));
                if ~any(d(i,:)==0)
                    Q(i,:)=ds_prime(i,:)/somd;
                else
                    for j=1:L
                        if d(i,j)>0
                            Q(i,j)=0;
                        else
                            Q(i,j)=1/sum(d(i,:)==0);
                        end
                    end
                end
            end
        end
        %fill empty clusters in Q
        if any(sum(Q)<numeps)
            dPV=zeros(n,L);
            for b=1:K
                dPV=dPV+Ps(:,b*ones1L).*V(b*onesn1,:);
            end
            dPV=dPV(:);
            dPVreplm=reshape(dPV(:,ones1m),[n L*m]);
            dXQ=reshape(sum(dPVreplm.*XreplL,1),[L m]);
            d=-2*dXQ'+XreplLsquared+onesm1*(sum(Ps,1)*V.^2);
            while any(sum(Q)<numeps)
                distance=sum(Q.*d,2);
                for l=1:L
                    Max=-1;
                    for i=1:m
                        if distance(i)>Max && all(-Q(i,:)+sum(Q)+(sum(Q)<numeps)>numeps)
                            maxindex=i;
                            Max=distance(i);
                        end
                    end
                    distance(maxindex)=0;
                    Q(maxindex,:)=0;
                    Q(maxindex,l)=1;
                end
            end
        end
        Qs=Q.^s;
        %update V
        V=Ps'*X*Qs./(sum(Ps)'*sum(Qs));
    end
    s=1+gamma*(s-1);
end
%perform the k-means algorithm to obtain a crisp partition
[vaf,P,Q]=kmeans(P,Q,X,m,n,K,L,variance);

